// sample3d.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// Sample 3-D OpenGL/GLUT program

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <GL/glut.h> // GLUT stuff - includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels
const double neardist = 1.0;  // Near & far clipping distances
const double fardist = 10.0;

// Torus
const double smallrad = 0.8;  // "Small" radius of torus
const double bigrad = 2.3;    // "Big" radius of torus
const int numsides = 40;      // Number of "sides" in torus
const int numrings = 50;      // Number of "rings" in torus


// setmaterial_main
// Sets up "main" material
void setmaterial_main()
{
   GLfloat paintcolor[] = { 0.9, 0.2, 0.5, 1.0 };
   GLfloat specularcolor[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat shininess[] = { 50. };
   glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, paintcolor);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularcolor);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // Draw a torus
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
      glTranslated(0., 0., -5.);
      setmaterial_main();
      glColor3d(0.9, 0.2, 0.5);      // Color - not used if lighting is on
      glutSolidTorus(smallrad, bigrad, numsides, numrings);
   glPopMatrix();

   glFlush();
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // We set up coordinate system so that aspect ratios are always correct,
   //  and the region from -1..1 in x & y always just fits in the viewport.
   if (w > h)
   {
      glFrustum(-double(w)/h, double(w)/h, -1., 1., neardist, fardist);
      // You may want to use gluPerspective here
   }
   else
   {
      glFrustum(-1., 1., -double(h)/w, double(h)/w, neardist, fardist);
   }

   glMatrixMode(GL_MODELVIEW);  // Always go back to modelview mode
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
   // General
   glClearColor(0.3, 0.4, 0.5, 0.0);  // Blue-gray background
   glEnable(GL_DEPTH_TEST);           // 3-D so we need HSR

   // Set up lighting
   GLfloat light0_position[] = { -1.0, 1.5, 2.0, 0.0 };
   GLfloat light0_maincolor[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat light0_ambientcolor[] = { 0.0, 0.0, 0.0, 1.0 };
   glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_maincolor);
   glLightfv(GL_LIGHT0, GL_SPECULAR, light0_maincolor);
   glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambientcolor);
   glEnable(GL_LIGHT0);

   GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 0.0 };  // = default
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
   glEnable(GL_LIGHTING);

   // Documentation
   cout << "Sample 3-D OpenGL/GLUT Program" << endl;
   cout << "by Glenn G. Chappell" << endl;
   cout << "for CS 381, Fall 2003" << endl;
   cout << endl;
   cout << "Kill window to quit." << endl;
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - Sample 3-D Program (kill window to quit)");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutIdleFunc(idle);

   // Do something
   glutMainLoop();

   return 0;
}
